@@ -1,11 +1,11 @@
 /***************************************************************************
-    copyright            : (C) 2002, 2003 by Scott Wheeler
+    copyright            : (C) 2002 - 2008 by Scott Wheeler
     email                : wheeler@kde.org
  ***************************************************************************/
 
 /***************************************************************************
  *   This library is free software; you can redistribute it and/or modify  *
- *   it  under the terms of the GNU Lesser General Public License version  *
+ *   it under the terms of the GNU Lesser General Public License version   *
  *   2.1 as published by the Free Software Foundation.                     *
  *                                                                         *
  *   This library is distributed in the hope that it will be useful, but   *
@@ -17,13 +17,21 @@
  *   License along with this library; if not, write to the Free Software   *
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
  *   USA                                                                   *
+ *                                                                         *
+ *   Alternatively, this file is available under the Mozilla Public        *
+ *   License Version 1.1.  You may obtain a copy of the License at         *
+ *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
+#ifndef HAVE_ZLIB
 #include <config.h>
+#endif
 
 #include <tdebug.h>
 
 #include "id3v2framefactory.h"
+#include "id3v2synchdata.h"
+#include "id3v1genres.h"
 
 #include "frames/attachedpictureframe.h"
 #include "frames/commentsframe.h"
@@ -31,6 +39,11 @@
 #include "frames/textidentificationframe.h"
 #include "frames/uniquefileidentifierframe.h"
 #include "frames/unknownframe.h"
+#include "frames/generalencapsulatedobjectframe.h"
+#include "frames/urllinkframe.h"
+#include "frames/unsynchronizedlyricsframe.h"
+#include "frames/popularimeterframe.h"
+#include "frames/privateframe.h"
 
 using namespace TagLib;
 using namespace ID3v2;
@@ -44,6 +57,12 @@ public:
 
   String::Type defaultEncoding;
   bool useDefaultEncoding;
+
+  template <class T> void setTextEncoding(T *frame)
+  {
+    if(useDefaultEncoding)
+      frame->setTextEncoding(defaultEncoding);
+  }
 };
 
 FrameFactory *FrameFactory::factory = 0;
@@ -66,13 +85,25 @@ Frame *FrameFactory::createFrame(const ByteVector &data, bool synchSafeInts) con
 
 Frame *FrameFactory::createFrame(const ByteVector &data, uint version) const
 {
+  Header tagHeader;
+  tagHeader.setMajorVersion(version);
+  return createFrame(data, &tagHeader);
+}
+
+Frame *FrameFactory::createFrame(const ByteVector &origData, Header *tagHeader) const
+{
+  ByteVector data = origData;
+  uint version = tagHeader->majorVersion();
   Frame::Header *header = new Frame::Header(data, version);
   ByteVector frameID = header->frameID();
 
   // A quick sanity check -- make sure that the frameID is 4 uppercase Latin1
   // characters.  Also make sure that there is data in the frame.
 
-  if(!frameID.size() == (version < 3 ? 3 : 4) || header->frameSize() <= 0) {
+  if(!frameID.size() == (version < 3 ? 3 : 4) ||
+     header->frameSize() <= uint(header->dataLengthIndicator() ? 4 : 0) ||
+     header->frameSize() > data.size())
+  {
     delete header;
     return 0;
   }
@@ -84,6 +115,14 @@ Frame *FrameFactory::createFrame(const ByteVector &data, uint version) const
     }
   }
 
+  if(version > 3 && (tagHeader->unsynchronisation() || header->unsynchronisation())) {
+    // Data lengths are not part of the encoded data, but since they are synch-safe
+    // integers they will be never actually encoded.
+    ByteVector frameData = data.mid(Frame::Header::size(version), header->frameSize());
+    frameData = SynchData::decode(frameData);
+    data = data.mid(0, Frame::Header::size(version)) + frameData;
+  }
+
   // TagLib doesn't mess with encrypted frames, so just treat them
   // as unknown frames.
 
@@ -99,10 +138,14 @@ Frame *FrameFactory::createFrame(const ByteVector &data, uint version) const
   }
 
   if(!updateFrame(header)) {
-    delete header;
-    return 0;
+    header->setTagAlterPreservation(true);
+    return new UnknownFrame(data, header);
   }
 
+  // updateFrame() might have updated the frame ID.
+
+  frameID = header->frameID();
+
   // This is where things get necissarily nasty.  Here we determine which
   // Frame subclass (or if none is found simply an Frame) based
   // on the frame ID.  Since there are a lot of possibilities, that means
@@ -111,12 +154,16 @@ Frame *FrameFactory::createFrame(const ByteVector &data, uint version) const
   // Text Identification (frames 4.2)
 
   if(frameID.startsWith("T")) {
+
     TextIdentificationFrame *f = frameID != "TXXX"
       ? new TextIdentificationFrame(data, header)
       : new UserTextIdentificationFrame(data, header);
 
-    if(d->useDefaultEncoding)
-      f->setTextEncoding(d->defaultEncoding);
+    d->setTextEncoding(f);
+
+    if(frameID == "TCON")
+      updateGenre(f);
+
     return f;
   }
 
@@ -124,8 +171,7 @@ Frame *FrameFactory::createFrame(const ByteVector &data, uint version) const
 
   if(frameID == "COMM") {
     CommentsFrame *f = new CommentsFrame(data, header);
-    if(d->useDefaultEncoding)
-      f->setTextEncoding(d->defaultEncoding);
+    d->setTextEncoding(f);
     return f;
   }
 
@@ -133,12 +179,19 @@ Frame *FrameFactory::createFrame(const ByteVector &data, uint version) const
 
   if(frameID == "APIC") {
     AttachedPictureFrame *f = new AttachedPictureFrame(data, header);
-    if(d->useDefaultEncoding)
-      f->setTextEncoding(d->defaultEncoding);
+    d->setTextEncoding(f);
     return f;
   }
 
-  // Relative Volume Adjustment (frames 4.11)
+  // ID3v2.2 Attached Picture
+
+	if(frameID == "PIC") {
+    AttachedPictureFrame *f = new AttachedPictureFrameV22(data, header);
+    d->setTextEncoding(f);
+    return f;
+  }
+
+	// Relative Volume Adjustment (frames 4.11)
 
   if(frameID == "RVA2")
     return new RelativeVolumeFrame(data, header);
@@ -148,6 +201,46 @@ Frame *FrameFactory::createFrame(const ByteVector &data, uint version) const
   if(frameID == "UFID")
     return new UniqueFileIdentifierFrame(data, header);
 
+  // General Encapsulated Object (frames 4.15)
+
+  if(frameID == "GEOB") {
+    GeneralEncapsulatedObjectFrame *f = new GeneralEncapsulatedObjectFrame(data, header);
+    d->setTextEncoding(f);
+    return f;
+  }
+
+  // URL link (frames 4.3)
+
+  if(frameID.startsWith("W")) {
+    if(frameID != "WXXX") {
+      return new UrlLinkFrame(data, header);
+    }
+    else {
+      UserUrlLinkFrame *f = new UserUrlLinkFrame(data, header);
+      d->setTextEncoding(f);
+      return f;
+    }
+  }
+
+  // Unsynchronized lyric/text transcription (frames 4.8)
+
+  if(frameID == "USLT") {
+    UnsynchronizedLyricsFrame *f = new UnsynchronizedLyricsFrame(data, header);
+    if(d->useDefaultEncoding)
+      f->setTextEncoding(d->defaultEncoding);
+    return f;
+  }
+
+  // Popularimeter (frames 4.17)
+
+  if(frameID == "POPM")
+    return new PopularimeterFrame(data, header);
+
+  // Private (frames 4.27)
+
+  if(frameID == "PRIV")
+    return new PrivateFrame(data, header);
+
   return new UnknownFrame(data, header);
 }
 
@@ -208,7 +301,6 @@ bool FrameFactory::updateFrame(Frame::Header *header) const
     convertFrame("IPL", "TIPL", header);
     convertFrame("MCI", "MCDI", header);
     convertFrame("MLL", "MLLT", header);
-    convertFrame("PIC", "APIC", header);
     convertFrame("POP", "POPM", header);
     convertFrame("REV", "RVRB", header);
     convertFrame("SLT", "SYLT", header);
@@ -266,14 +358,14 @@ bool FrameFactory::updateFrame(Frame::Header *header) const
        frameID == "RVAD" ||
        frameID == "TIME" ||
        frameID == "TRDA" ||
-       frameID == "TSIZ")
+       frameID == "TSIZ" ||
+       frameID == "TDAT")
     {
       debug("ID3v2.4 no longer supports the frame type " + String(frameID) +
             ".  It will be discarded from the tag.");
       return false;
     }
 
-    convertFrame("TDAT", "TDRC", header);
     convertFrame("TORY", "TDOR", header);
     convertFrame("TYER", "TDRC", header);
 
@@ -307,3 +399,34 @@ void FrameFactory::convertFrame(const char *from, const char *to,
 
   header->setFrameID(to);
 }
+
+void FrameFactory::updateGenre(TextIdentificationFrame *frame) const
+{
+  StringList fields = frame->fieldList();
+  StringList newfields;
+
+  for(StringList::Iterator it = fields.begin(); it != fields.end(); ++it) {
+    String s = *it;
+    int end = s.find(")");
+
+    if(s.startsWith("(") && end > 0) {
+      // "(12)Genre"
+      String text = s.substr(end + 1);
+      int number = s.substr(1, end - 1).toInt();
+      if (number > 0 && number <= 255 && !(ID3v1::genre(number) == text))
+        newfields.append(s.substr(1, end - 1));
+      if (!text.isEmpty())
+        newfields.append(text);
+    }
+    else {
+      // "Genre" or "12"
+      newfields.append(s);
+    }
+  }
+
+  if(newfields.isEmpty())
+    fields.append(String::null);
+
+  frame->setText(newfields);
+
+}
